// Copyright (C) 2012 Donald A. Graft, All Rights Reserved

#include <stdio.h>
#include <math.h>
#include <time.h>
#include "utils.h"

int main(int argc, char *argv[])
{
	unsigned int i, j, count, detected_pairs, anticorrelations, doubles, misses;
	int a_up, a_down, b_up, b_down;
	double theta1, theta2, A, B, threshA, threshB, decoherence;
	double noise_amplitude, noise_variance, efficiency;
	double fa, fb;
	unsigned int singles_a, singles_b;
	unsigned int doubles_a, doubles_b;
	FILE *fp;

	fopen_s(&fp, "f:\\tmp\\Correlations\\spread.txt", "w");
	if (fp == 0)
	{
		printf("Cannot open data file.\n");
		return 0;
	}

	seed_ran1((int) time(NULL));

	threshA = .50;
	threshB = .75;
	decoherence = 0.2;
	noise_amplitude = 0.3;
	noise_variance = 0.2;
	efficiency = 0.5;
	count = 30000;

	for (j = 0; j <= 180; j+=2)
	{
		A = 0; 
//		A = PI / 4;
		B = j * PI/180;
		anticorrelations = 0;
		singles_a = singles_b = 0;
		doubles_a = doubles_b = 0;
		detected_pairs = doubles = misses = 0;

		for (i = 0; i < count; i++)
		{
			// Generate "entangled" photons.
			theta1 = (2 * PI * ran1());
			theta2 = theta1 + PI/2 + ran2(decoherence);
			if (theta2 > 2 * PI)
				theta2 -= 2 * PI;

			// Calculate side A detections.
			a_up = a_down = 0;
			if (ran1() < efficiency)
			{
				fa = cos(A - theta1) * cos(A - theta1) + noise_amplitude * ran2(noise_variance);
				if (fa > threshA)
				{
					a_up = 1;
				}
				fa = 1 - fa;
				if (fa > threshA)
				{
					a_down = 1;
				}
				if (a_up ^ a_down)
					singles_a++;
				else if (a_up && a_down)
					doubles_a++;
			}

			// Calculate side B detections.
			b_up = b_down = 0;
			if (ran1() < efficiency)
			{
				fb = cos(B - theta2) * cos(B - theta2) + noise_amplitude * ran2(noise_variance);
				if (fb > threshB)
				{
					b_up = 1;
				}
				fb = 1 - fb;
				if (fb > threshB)
				{
					b_down = 1;
				}
				if (b_up ^ b_down)
					singles_b++;
				else if (b_up && b_down)
					doubles_b++;
			}

			// Correlate.
			if ((a_up && a_down) || (b_up && b_down))
			{
				// Double detection.
				doubles++;
			}
			else if ((!a_up && !a_down) || (!b_up && !b_down))
			{
				// Missed detection.
				misses++;
			}
			else
			{
				detected_pairs++; 
				if ((a_up && b_down) || (a_down && b_up))
					anticorrelations++;
			}
		}
		printf("%d,%f,%d,%d,%d\n", j, (double) anticorrelations / detected_pairs, detected_pairs, doubles, misses);
		fprintf(fp, "%d,%f,%f,%f,%f,%f,%f\n",
				j, (double) anticorrelations / detected_pairs, (double) detected_pairs, (double) singles_a, (double) singles_b, (double) doubles_a, (double) doubles_b);
	}
	fclose(fp);
	return(0);
}
