#include <math.h>
#include "utils.h"

#define IA 16807
#define IM 2147483647
#define AM ((float) (1.0/IM))
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS ((float)1.2e-7)
#define RNMX ((float) (1.0-EPS))

long Idum;

void seed_ran1(int seed)
{
	float ran1(void);

	Idum = -seed; // Arbitrary seed.
	ran1();
}

// return uniform random deviate in the range 0.0 to 1.0.
float ran1(void)
{
	int j;
	long k;
	static long iy = 0;
	static long iv[NTAB];
	float temp;
	long *idum = &Idum;

	if (*idum <= 0 || !iy)
	{
		if (-(*idum) < 1) *idum = 1;
		else *idum = -(*idum);
		for (j = NTAB + 7; j >= 0; j--)
		{
			k = (*idum)/IQ;
			*idum = IA * (*idum - k * IQ) - IR * k;
			if (*idum < 0) *idum += IM;
			if (j < NTAB) iv[j] = *idum;
		}
		iy = iv[0];
	}
	k = (*idum) / IQ;
	*idum = IA * (*idum - k * IQ) - IR * k;
	if (*idum < 0) *idum += IM;
	j = iy / NDIV;
	iy = iv[j];
	iv[j] = *idum;
	if ((temp = AM * iy) > RNMX) return RNMX;
	else return temp;
}

// Return gaussian random deviate with mean 0.0 and specified variance.
double ran2(double variance)
{
	static int have_deviate = 0;
	static double u1, u2;
	double x1, x2, w;

	if (have_deviate)
	{
		have_deviate = 0;
		return u2 * variance;
	}
	else
	{
		do
		{
			x1 = 2.0 * ran1() - 1.0;
			x2 = 2.0 * ran1() - 1.0;
			w = x1 * x1 + x2 * x2;
		} while ( w >= 1.0 );
		w = sqrt ((-2.0 * log (w))/w);
		u1 = x1 * w;
		u2 = x2 * w;
		have_deviate = 1;
		return u1 * variance;
	}
}
